/*	$OpenBSD: curses.h,v 1.5 2019/01/25 00:19:25 millert Exp $	*/

/*
 * Placed in the public domain by Todd C. Miller <millert@openbsd.org>
 * on June 17, 2003.
 */

#ifndef _CURSES_H_
#define _CURSES_H_

#include <ncurses.h>

#endif /* _CURSES_H_ */
