/*	$OpenBSD: setjmp.h,v 1.2 2011/03/23 16:54:35 pirofti Exp $	*/

/*
 * machine/setjmp.h: machine dependent setjmp-related information.
 */
#ifndef _M88K_SETJMP_H_
#define _M88K_SETJMP_H_
#define	_JBLEN	21		/* size, in longs, of a jmp_buf */
#endif /* _M88K_SETJMP_H_ */
