/*	$OpenBSD: tcdevs_data.h,v 1.8 2006/03/13 22:00:42 miod Exp $	*/

/*
 * THIS FILE AUTOMATICALLY GENERATED.  DO NOT EDIT.
 *
 * generated from:
 *	OpenBSD: tcdevs,v 1.7 2002/05/03 20:27:44 miod Exp 
 */

struct tc_knowndev tc_knowndevs[] = {
	{
	    "AV01B-AA",
	    "DecAudio \"LoFi\" audio/isdn"
	},
	{
	    "AV300-AA",
	    "Video capture option card"
	},
	{
	    "DGLTA-FA",
	    "DGLTA ATM"
	},
	{
	    "FORE_ATM",
	    "Fore TCA-100 ATM"
	},
	{
	    "KZTSA-AA",
	    "TZA FWD SCSI"
	},
	{
	    "OTTO    ",
	    "DEC SRC \"OTTO\" ATM"
	},
	{
	    "PMAD-AA ",
	    "LANCE Ethernet"
	},
	{
	    "PMAD-AB ",
	    "LANCE Ethernet"
	},
	{
	    "PMAF-AA ",
	    "DEC FDDIcontroller 700 (DEFZA; fiber optic)"
	},
	{
	    "PMAF-FA ",
	    "DEFTA FDDI"
	},
	{
	    "PMAG-AA ",
	    "Monochrome Frame Buffer"
	},
	{
	    "PMAG-BA ",
	    "Color Frame Buffer"
	},
	{
	    "PMAG-CA ",
	    "2D Graphics (PX 2DA)"
	},
	{
	    "PMAG-CB ",
	    "2D Graphics (PX 2DA)"
	},
	{
	    "PMAG-DA ",
	    "3D Graphics (PXG LM-3DA)"
	},
	{
	    "PMAG-DB ",
	    "3D Graphics (PXG LM-3DA)"
	},
	{
	    "PMAG-EB ",
	    "3D Graphics (PXG MID-3DA)"
	},
	{
	    "PMAG-FA ",
	    "3D Graphics (PXG HE-3DA)"
	},
	{
	    "PMAG-FB ",
	    "3D Graphics (PXG HE+3DA)"
	},
	{
	    "PMAGB-BA",
	    "Smart Frame Buffer (HX8)"
	},
	{
	    "PMAGB-FA",
	    "3D Graphics (PXG HE+3DA)"
	},
	{
	    "PMAGB-FB",
	    "3D Graphics (PXG HE+3DA)"
	},
	{
	    "PMAGC-AA",
	    "2D Graphics (PV-LO)"
	},
	{
	    "PMAGC-BA",
	    "2D Graphics (PV-MID)"
	},
	{
	    "PMAGD-AA",
	    "Smart Frame Buffer Plus, 8bpp (ZLX-E1)"
	},
	{
	    "PMAGD-BA",
	    "Smart Frame Buffer Plus, 32bpp (ZLX-E2)"
	},
	{
	    "PMAGD-CA",
	    "Smart Frame Buffer Plus, 32bpp (ZLX-E3)"
	},
	{
	    "PMAZ-AB ",
	    "53c94 SCSI Controller"
	},
	{
	    "KWS_TD  ",
	    "Kubota Denali"
	},
	{
	    "PMABV-AA",
	    "VME Adapter"
	},
	{
	    "PMAG-DV ",
	    "Maxine Color Frame Buffer"
	},
	{
	    "PMAG-JA ",
	    "24-plane True Color Frame Buffer (TX)"
	},
	{
	    "PMAGD   ",
	    "Smart Frame Buffer Plus, unknown bpp"
	},
	{
	    "PMAP-AA ",
	    "Prestoserve"
	},
	{
	    "PMAT-AA ",
	    "DEC TurboChannel Token Ring Controller"
	},
	{
	    "PMAZ-AA ",
	    "53c94 SCSI Controller"
	},
	{
	    "PMAZ-DS ",
	    "53c94 TCDS SCSI Controller (baseboard)"
	},
	{
	    "PMAZ-FS ",
	    "53c94 TCDS Fast SCSI Controller (baseboard)"
	},
	{
	    "PMAZB-AA",
	    "53c94 TCDS SCSI Controller option card"
	},
	{
	    "PMAZB-AB",
	    "53c94 TCDS SCSI Controller option card"
	},
	{
	    "PMAZC-AA",
	    "53c94 TCDS Fast SCSI Controller option card"
	},
	{
	    "PMTNV-AA",
	    "Non-volatile RAM option card"
	},
	{
	    "T1D4PKT ",
	    "DECWRL Turbochannel T1"
	},
	{
	    "T3PKT   ",
	    "DECWRL Turbochannel T3"
	},
	{ NULL, NULL }
};
